/*
 * cplugin.h
 * edited by whiteball
 * 
 * original(csaori.h)
 * written by Ukiya http://ukiya.sakura.ne.jp/
 * based by тl "gethwnd.dll"
 */
#pragma once

#define PLUGINAPI extern "C" __declspec(dllexport)

#define PLUGINCDECL __cdecl

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <string>
#include <vector>
#include <map>
#include <queue>

#include <boost/unordered_map.hpp>
#include <boost/function.hpp>

//PLUGIN INTERFACES
PLUGINAPI BOOL PLUGINCDECL load(HGLOBAL h,long len);
PLUGINAPI BOOL PLUGINCDECL unload();
PLUGINAPI HGLOBAL PLUGINCDECL request(HGLOBAL h,long* len);

//DLLMain
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved);

#define PLUGIN_VERSIONSTRING_20		L"PLUGIN/2.0"
#define PLUGIN_VERSIONSTRING		PLUGIN_VERSIONSTRING_20
#define PLUGIN_SIZEOFVERSIONSTRING	(sizeof(PLUGIN_VERSIONSTRING) - 1)
#define PLUGIN_TARGET_ALL_GOHST		L"__SYSTEM_ALL_GHOST__"

class CPLUGIN;
class CPLUGINInput;
class CPLUGINOutput;

typedef wchar_t char_t;
typedef std::basic_string<char_t> string_t;
typedef boost::unordered_map<string_t,boost::function<void(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&)> > function_table_t;

typedef enum {CHARSET_Shift_JIS,CHARSET_ISO_2022_JP,CHARSET_EUC_JP,CHARSET_UTF_8} CHARSET;
typedef enum {PLUGINRESULT_OK=200,PLUGINRESULT_NO_CONTENT=204,PLUGINRESULT_BAD_REQUEST=400,PLUGINRESULT_INTERNAL_SERVER_ERROR=500} PLUGINRESULT;
typedef enum {PLUGINEVENTOPTION_NONE=0,PLUGINEVENTOPTION_NOTIFY=1} PLUGINEVENTOPTION;
typedef enum {PLUGINSCRIPTOPTION_NONE=0,PLUGINSCRIPTOPTION_NOTRANSLATE=1,PLUGINSCRIPTOPTION_NOBREAK=2} PLUGINSCRIPTOPTION;

//global functions
namespace PLUGIN_FUNC{
	string_t numToString(unsigned num, unsigned int base=10, char_t sign=L'\0');
	string_t intToString(int num, unsigned int base=10);
	string_t::size_type  getLine(string_t &, const string_t &, string_t::size_type);
	string_t getResultString(int);
	void splitString(const string_t &input, const string_t &split, std::vector<string_t> &output);
	void registerEvent(const string_t &event_name, const std::vector<string_t> &refernce);

	std::string UnicodeToMultiByte(const std::wstring& Source, UINT CodePage=CP_OEMCP, DWORD Flags=0);
	std::wstring MultiByteToUnicode(const std::string& Source, UINT CodePage=CP_OEMCP, DWORD Flags=0);
	UINT CHARSETtoCodePage(CHARSET cset);
	std::wstring CHARSETtoString(CHARSET cset);
}

//Classes
class CPLUGINInput{
public:
	CHARSET charset;
	string_t cmd;
	string_t sender;
	string_t id;
	std::vector<string_t> references;

	bool parseString(const string_t &src);
};

class CPLUGINOutput{
public:
	CHARSET charset;
	PLUGINRESULT result_code;
	string_t target;
	string_t event;
	PLUGINEVENTOPTION event_opt;
	string_t script;
	PLUGINSCRIPTOPTION script_opt;
	std::vector<string_t> references;
	
	string_t toString();
	string_t toStringForVersion(const string_t &version);
};

class CPLUGINListInfo{
public:
	string_t name;
	string_t id;
};

class CPLUGINGhostInfo{
public:
	std::vector<HWND> hwnd_list;
	string_t ghost_name;
	string_t shell_name;
	string_t id;
	string_t path;

	CPLUGINGhostInfo(const std::vector<string_t> &reference);
	CPLUGINGhostInfo(const CPLUGINGhostInfo &obj);
	CPLUGINGhostInfo &operator =(const CPLUGINGhostInfo &obj);

	void init(const std::vector<string_t> &reference);
	// for debug
	string_t toString(void)const;
};

class CPLUGINQueueEvent{
public:
	string_t event_name;
	std::vector<string_t> reference;

	CPLUGINQueueEvent(const string_t &evt,const std::vector<string_t> &ref);
	CPLUGINQueueEvent(const CPLUGINQueueEvent &obj);
};

class CPLUGIN{
	friend void installedplugin__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void ghostpathlist__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void balloonpathlist__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void headlinepathlist__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void pluginpathlist__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void OnGhostInfoUpdate__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void OnGhostBoot__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void OnGhostExit__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);
	friend void OnSecondChange__(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&);

	friend void PLUGIN_FUNC::registerEvent(const string_t &, const std::vector<string_t>&);

public:
	CPLUGIN();

private:
	string_t module_path;
	string_t version;
	std::vector<CPLUGINListInfo> plugin_list;
	std::vector<string_t> ghost_path_list;
	std::vector<string_t> balloon_path_list;
	std::vector<string_t> headline_path_list;
	std::vector<string_t> plugin_path_list;
	std::vector<CPLUGINGhostInfo> ghost_info_list;

	boost::unordered_map<string_t,boost::function<void(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&)> > function_table;
	boost::unordered_map<string_t,boost::function<void(CPLUGIN&,const CPLUGINInput&,CPLUGINOutput&)> > system_function_table;
	
	std::queue<CPLUGINQueueEvent> event_queue;

public:
	void setModulePath(const std::string &str);
	const string_t &getModulePath(void)const;
	const std::vector<CPLUGINListInfo> &getPluginList(void)const;
	const std::vector<string_t> &getGhostPathList(void)const;
	const std::vector<string_t> &getBalloonPathList(void)const;
	const std::vector<string_t> &getHeadlinePathList(void)const;
	const std::vector<string_t> &getPluginPathList(void)const;
	const std::vector<CPLUGINGhostInfo> &getGhostInfoList(void)const;
	
	std::string request(const std::string &req);

	//ȉׂ֐
	bool unload();
	bool load();
};
